-----------------------------------------------------------------------------------------------
-- Client Lua Script for ChatWhoIs
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- ChatWhoIs Module Definition
-----------------------------------------------------------------------------------------------
local ChatWhoIs = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ChatWhoIs:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	ChatWhoIs.sortField = "name"

    return o
end

function ChatWhoIs:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
	Apollo.RegisterEventHandler("WhoResponse", "OnWhoResponse", self)
end
 

-----------------------------------------------------------------------------------------------
-- ChatWhoIs OnLoad
-----------------------------------------------------------------------------------------------
function ChatWhoIs:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("ChatWhoIs.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- ChatWhoIs OnDocLoaded
-----------------------------------------------------------------------------------------------
function ChatWhoIs:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "ChatWhoIsForm", nil, self)
		self.list = self.wndMain:FindChild("Content")
		
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)


		-- Do additional Addon initialization here
	end
end

-----------------------------------------------------------------------------------------------
-- ChatWhoIs Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
function ChatWhoIs:OnWhoResponse(tResults)
	local count = 0
	result = {}
	
	for k, v in pairs(tResults) do
		count = count + 1
		
		local race = string.match(v.strRace, '[a-zA-Z]*')
		
		result[k] = {name = v.strName, level = v.nLevel, race = race, class = v.strClass, location = v.strZone, path = v.strPath}
	end
	
	self.result = result;
	
	if count <= 2 then
		for k, v in pairs(result) do
			Print("[" .. v.name .. "] Stufe " .. v.level .. " - " .. v.race .. ", " .. v.class .. " - " .. v.location)
		end

		if count == 1 then
			Print(count .. " Player found")
		else
			Print(count .. " Players found")
		end
	else
		self.wndMain:FindChild("PlayersFoundCount"):SetText(count)
	
		self:FillResults()
		self.wndMain:Show(true)
	end
end

function ChatWhoIs.sort(objA, objB)
	return objA[ChatWhoIs.sortField] < objB[ChatWhoIs.sortField]
end

function ChatWhoIs:FillResults()
	for i, child in ipairs(self.list:GetChildren()) do
		child:Destroy()
	end
	
	table.sort(self.result, ChatWhoIs.sort)
	for k, v in ipairs(self.result) do
		local listEntry = Apollo.LoadForm(self.xmlDoc, "WhoListItem", self.list, self)
		listEntry:FindChild("txtName"):SetText(v.name)
		listEntry:FindChild("txtLevel"):SetText(v.level)
		listEntry:FindChild("txtRace"):SetText(v.race)
		listEntry:FindChild("txtClass"):SetText(v.class)
		listEntry:FindChild("txtPath"):SetText(v.path)
		listEntry:FindChild("txtLocation"):SetText(v.location)
	end
	self.list:ArrangeChildrenVert()
end

-----------------------------------------------------------------------------------------------
-- ChatWhoIsForm Functions
-----------------------------------------------------------------------------------------------
function ChatWhoIs:OnWhoSortToggle( wndHandler, wndControl, eMouseButton )
	local name = wndControl:GetName()
	
	if name == "SortBtnName" then ChatWhoIs.sortField = "name" end
	if name == "SortBtnLevel" then ChatWhoIs.sortField = "level" end
	if name == "SortBtnRace" then ChatWhoIs.sortField = "race" end
	if name == "SortBtnClass" then ChatWhoIs.sortField = "class" end
	if name == "SortBtnPath" then ChatWhoIs.sortField = "path" end
	if name == "SortBtnLocation" then ChatWhoIs.sortField = "location" end
	
	self:FillResults()
end

---------------------------------------------------------------------------------------------------
-- WhoListItem Functions
---------------------------------------------------------------------------------------------------

function ChatWhoIs:OnMouseButtonDown( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation )
end

-----------------------------------------------------------------------------------------------
-- ChatWhoIs Instance
-----------------------------------------------------------------------------------------------
local ChatWhoIsInst = ChatWhoIs:new()
ChatWhoIsInst:Init()
